<?php

namespace App\Repositories;

use App\Dto\EvidenciaDto;
use App\Dto\EvidenciaRegistroDto;
use App\Dto\EvidenciaRetoDto;
use App\Dto\ResponseTotalEvidenciaRegistroDto;
use App\Enums\EstatusEvidencia;
use App\Interfaces\EvidenciaInterface;
use App\Models\Evidencia;
use App\Models\EvidenciaRegistro;
use App\Models\EvidenciaReto;

class EvidenciaRepository implements EvidenciaInterface
{

    public function allEvidencias()
    {
        $evidencias = Evidencia::with(['files', 'retos', 'participante', 'registros_evidencias.producto'])->get();
        return $evidencias;
    }

    public function detailEvidencia(string $idEvidencia)
    {
        $evidencia = Evidencia::with(['files', 'retos', 'participante', 'usuario', 'registros_evidencias.producto'])
            ->where("id", $idEvidencia)
            ->first();
        return $evidencia;
    }

    public function createEvidencia(EvidenciaDto $evidenciaDto)
    {
        $evidenciaNew = Evidencia::create([
            "fecha_captura" => $evidenciaDto->fechaCaptura,
            "comentario" => $evidenciaDto->comentario,
            "user_id" => $evidenciaDto->userId,
            "participante_id" => $evidenciaDto->participanteId,
            "file_id" => $evidenciaDto->fileId,
        ]);
        return $evidenciaNew;
    }

    public function updateEvidenciaEstatus(EvidenciaDto $evidenciaDto, string $idEvidencia)
    {
        $evidencia = Evidencia::findOrFail($idEvidencia);
        $evidencia->update([
            "estatus" =>  $evidenciaDto->estatus
        ]);
        $evidenciaUpdateDto = new EvidenciaDto();
        $evidenciaUpdateDto->id = $evidencia->id;
        $evidenciaUpdateDto->fechaCaptura = $evidencia->fecha_captura;
        $evidenciaUpdateDto->participanteId = $evidencia->participante_id;
        return $evidenciaUpdateDto;
    }

    public function createEvidenciaRegistro(EvidenciaRegistroDto $evidenciaRegistroDto)
    {
        $newRegistro = EvidenciaRegistro::create([
            "cantidad" => $evidenciaRegistroDto->cantidad,
            "monto" => $evidenciaRegistroDto->monto,
            "precio_producto" => $evidenciaRegistroDto->precioProducto,
            "producto_id" => $evidenciaRegistroDto->idProducto,
            "evidencia_id" => $evidenciaRegistroDto->evidenciaId
        ]);
        return $newRegistro;
    }

    public function createReto(EvidenciaRegistroDto $evidenciaRegistroDto)
    {
        $newRegistro = EvidenciaRegistro::create([
            "cantidad" => $evidenciaRegistroDto->cantidad,
            "monto" => $evidenciaRegistroDto->monto,
            "precio_producto" => $evidenciaRegistroDto->precioProducto,
            "producto_id" => $evidenciaRegistroDto->idProducto,
            "evidencia_id" => $evidenciaRegistroDto->evidenciaId
        ]);
        return $newRegistro;
    }

    public function createEvidenciaReto(EvidenciaRetoDto $evidenciaRetoDto)
    {
        $newRegistro = EvidenciaReto::create([
            'reto_id' => $evidenciaRetoDto->idReto,
            'evidencia_id' => $evidenciaRetoDto->idEvidencia
        ]);
        return $newRegistro;
    }

    public function findEvidenciasByPlayer(string $idPlayer)
    {
        $evidencias = Evidencia::where('participante_id', $idPlayer)
            ->with('participante', 'files', 'retos', 'registros_evidencias.producto')
            ->take(200)
            ->orderBy('fecha_captura', 'desc')
            ->get();
        return $evidencias;
    }

    public function findEvidenciasByUser(string $idUser)
    {
        $evidencias = Evidencia::with('registros_evidencias')
            ->where('user_id', $idUser)
            ->orderBy('fecha_captura', 'desc')
            ->get();
        return $evidencias;
    }

    public function totaAmountsRegisterEvidencias(string $idPlayer, string $retoId)
    {
        $evidenciaQuery = EvidenciaRegistro::with(['evidencia_detail'])
            ->whereHas('evidencia_detail', function ($query) use ($idPlayer) {
                $query->where('estatus', EstatusEvidencia::APROBADO);
                $query->where('participante_id', $idPlayer);
            })
            ->whereHas('evidencia_detail.retos', function ($query) use ($retoId) {
                $query->where('reto_id', $retoId);
            });
        $totalData = $evidenciaQuery->selectRaw('SUM(cantidad) as total_cantidad, SUM(monto) as total_monto')
            ->first();
        $cantidadTotal = $totalData->total_cantidad;
        $cantidadMonto = $totalData->total_monto;
        $result = new ResponseTotalEvidenciaRegistroDto($cantidadMonto, $cantidadTotal);
        return $result;
    }
}
