<?php

namespace App\Repositories;

use App\Dto\HistorialRecompensaDto;
use App\Dto\HistoryActionPlayerDto;
use App\Interfaces\HistorialRecompensaInterface;
use App\Models\HistorialRecompensas;
use Illuminate\Support\Collection;

class HistorialRecompensaRepository implements HistorialRecompensaInterface
{

    public function allHistorialRecompensas()
    {
        $recompensas = HistorialRecompensas::with(['nivel_reto.reto_detail', 'nivel_reto.insignias.recompensa.files', 'participante'])
            ->whereHas('nivel_reto.insignias.recompensa')
            ->get();
        return $recompensas;
    }

    public function findDetailRecompensa(string $idHistorialRecompensa)
    {
        $recompensa = HistorialRecompensas::with(['participante', 'nivel_reto.insignias.recompensa.files'])
            ->where("id", $idHistorialRecompensa)->first();
        return $recompensa;
    }

    public function findRecompensasHasInsigniasByPlayer(string $idPlayer)
    {
        $recompensas = HistorialRecompensas::with([
            'nivel_reto.reto_detail',
            'nivel_reto.insignias.recompensa.files', 'participante'
        ])
            ->whereHas('nivel_reto.insignias.recompensa')
            ->where('participante_id', $idPlayer)
            ->get();
        return $recompensas;
    }

    public function createHistorialRecompensa(HistorialRecompensaDto $historialRecompensaDto) {
        $historialNew = HistorialRecompensas::create([
            "participante_id" => $historialRecompensaDto->participanteId,
            "nivel_id" => $historialRecompensaDto->nivelId,
            "user_id" => $historialRecompensaDto->userId
        ]);
        return $historialNew;
    }

    public function updateHistorialRecompensa(HistorialRecompensaDto $historialRecompensaDto, string $idHistorialRecompensa) {
        $historialUpdate = HistorialRecompensas::findOrFail($idHistorialRecompensa);
        $historialUpdate->update([
            "fecha_entrega" => $historialRecompensaDto->fechaEntrega,
            "estatus" => $historialRecompensaDto->estatus,
            "user_id" => $historialRecompensaDto->userId
        ]);
        $newHistoryUpdate = new HistorialRecompensaDto();
        $newHistoryUpdate->fechaEntrega = $historialUpdate->fecha_entrega;
        $newHistoryUpdate->estatus = $historialUpdate->estatus;
        $newHistoryUpdate->userId = $historialUpdate->user_id;
        $newHistoryUpdate->participanteId = $historialUpdate->participante_id;
        $newHistoryUpdate->nivelId = $historialUpdate->nivel_id;
        return $newHistoryUpdate;
    }

    public function historyActionRecompensas(string $idPlayer){
        $recompensasQuery = HistorialRecompensas::join("nivel_retos", "historial_recompensas.nivel_id", "nivel_retos.id")
        ->join("retos", "nivel_retos.reto_id", "retos.id")
        ->where("historial_recompensas.participante_id", $idPlayer)
        ->where("historial_recompensas.estatus", "entregado")
        ->select("historial_recompensas.id", "historial_recompensas.fecha_entrega as fecha",
        "nivel_retos.puntos as puntos", "nivel_retos.nombre as nombre_nivel", "retos.nombre as nombre_reto")
        ->get();
        $datos = $recompensasQuery->map(function ($recompensa) {
            $newHistoryAction = new HistoryActionPlayerDto();
            $newHistoryAction->accion = "Ganaste";
            $newHistoryAction->cantidadPuntos = $recompensa->puntos;
            $newHistoryAction->descripcion = "$recompensa->nombre_reto - $recompensa->nombre_nivel";
            $newHistoryAction->fecha = $recompensa->fecha;
            return $newHistoryAction;
        });
        return new Collection($datos);
    }
}
