<?php

namespace App\Repositories;

use App\Dto\ProductDto;
use App\Interfaces\ProductInterface;
use App\Models\Producto;
use Illuminate\Support\Facades\DB;

class ProductRepository implements ProductInterface
{
    public function allProducts()
    {
        $productos = Producto::with('files')->get();
        return $productos;
    }

    public function findProduct(string $productId)
    {
        $producto = Producto::with('files')->where("id", $productId)->first();
        return $producto;
    }

    public function createProduct(ProductDto $productDto)
    {
        $newProduct = Producto::create([
            "nombre" => $productDto->nombre,
            "descripcion" => $productDto->descripcion,
            "user_id" => $productDto->userId,
            'file_id' => $productDto->fileId
        ]);
        return $newProduct;
    }

    public function updateProduct(ProductDto $productDto, int $idProduct)
    {
        $productUpdate = Producto::findOrFail($idProduct);
        $productUpdate->update([
            "nombre" => $productDto->nombre,
            "descripcion" => $productDto->descripcion,
            "estatus" => $productDto->estatus,
            'file_id' => $productDto->fileId
        ]);
        return $productUpdate;
    }

    public function deleteProduct(int $idProduct)
    {
        $produtDelete = Producto::findOrfail($idProduct);
        $produtDelete->delete();
    }

    public function findTotalVentaProductosByPlayer(string $idPlayer)
    {
        $totalVentaProducto = Producto::select(
            'productos.id',
            'productos.nombre',
            DB::raw('SUM(evidencia_registros.cantidad) as cantidad_total_acumulada'),
            DB::raw('SUM(evidencia_registros.monto) as monto_total_acumulada')
        )
            ->join('evidencia_registros', 'productos.id', '=', 'evidencia_registros.producto_id')
            ->join('evidencias', 'evidencia_registros.evidencia_id', '=', 'evidencias.id')
            ->where('evidencias.participante_id', $idPlayer)
            ->where('evidencias.estatus', 'aprobado')
            ->groupBy('productos.id', 'productos.nombre')
            ->get();
        return $totalVentaProducto;
    }

    public function findTotalVentaProductos()
    {
        $totalVentaProducto = Producto::select(
            'productos.id',
            'productos.nombre',
            DB::raw('SUM(evidencia_registros.cantidad) as cantidad_total_acumulada'),
            DB::raw('SUM(evidencia_registros.monto) as monto_total_acumulada')
        )
            ->join('evidencia_registros', 'productos.id', '=', 'evidencia_registros.producto_id')
            ->join('evidencias', 'evidencia_registros.evidencia_id', '=', 'evidencias.id')
            ->where('evidencias.estatus', 'aprobado')
            ->groupBy('productos.id', 'productos.nombre')
            ->get();
        return $totalVentaProducto;
    }
}
