<?php

namespace App\Services;

use App\Dto\Responses\RegistrationResponseDto;
use App\Dto\Responses\ResponseDataDto;
use App\Dto\Responses\ResponseUserDto;
use App\Dto\UserDto;
use App\Repositories\UserRepository;
use Illuminate\Support\Facades\Log;

class AuthService
{
    protected $userRepository;

    public function __construct(UserRepository $userRepository)
    {
        $this->userRepository = $userRepository;
    }

    public function isRoleAdmin(string $idUser){
        try {
            $isAdmin = $this->userRepository->isUserWithRolAdmin($idUser);
            return new ResponseDataDto(true, $isAdmin);
        } catch (\Exception $e) {
            Log::error('Error al verificar si tiene el rol administrador' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, false);
        }
    }

    public function updateUserById(UserDto $userDto, string $userId){
        try {
            $userUpdateResult = $this->userRepository->updateUser($userDto, $userId);
            $userDto->email = $userUpdateResult->email;
            return new ResponseUserDto(true, $userDto);
        } catch (\Exception $e) {
            Log::error('Error al actualizar el usuario ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseUserDto(false, null);
        }
    }

    public function getUserById(string $idUser){
        try {
            $userFind = $this->userRepository->findUser($idUser);
            return $userFind;
        } catch (\Exception $e) {
            Log::error('Error al consultar el usuario ' . $e->getMessage(), ['exception' => $e]);
            return null;
        }
    }

    public function registerUserRol(UserDto $userDto, string $rol){
        try {
            $newUser = $this->userRepository->createUserRol($userDto, $rol);
            $userDto->id = $newUser->id;
            return new RegistrationResponseDto(true, $userDto);
        } catch (\Exception $e) {
            Log::error('Error al registrar el usuario: ' . $e->getMessage(), ['exception' => $e]);
            return new RegistrationResponseDto(false, null);
        }
    }

    public function removeUserById(string $idUser){
        try {
            $this->userRepository->deleteUser($idUser);
            return true;
        } catch (\Exception $e) {
            Log::error('Error al eliminar el usuario: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }

    public function getUserByPasswordConfirm(string $email, bool $confirm){
        try {
            $userFind = $this->userRepository->findUserByConfirmPassword($email, $confirm);
            return $userFind;
        } catch (\Exception $e) {
            Log::error('Error al consultar si el usuario confirmo la contraseña ' . $e->getMessage(), ['exception' => $e]);
            return null;
        }
    }

    public function getPerfilUser(string $idUser)
    {
        try {
            $queryFilter = $this->userRepository->findPerfilUser($idUser);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar el perfil del usuario: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }
}
