<?php

namespace App\Services;

use App\Dto\EvidenciaDto;
use App\Dto\EvidenciaRegistroDto;
use App\Dto\EvidenciaRetoDto;
use App\Dto\NivelRetoDto;
use App\Dto\Responses\ResponseDataDto;
use App\Dto\Responses\ResponseEvidenciaDto;
use App\Dto\Responses\ResponseEvidenciaRegistroDto;
use App\Dto\Responses\ResponseValidSuccess;
use App\Repositories\EvidenciaRepository;
use Illuminate\Support\Facades\Log;

class EvidenciaService
{
    protected $evidenciaRepository;

    public function __construct(EvidenciaRepository $evidenciaRepository)
    {
        $this->evidenciaRepository = $evidenciaRepository;
    }

    public function getEvidenciasAll()
    {
        try {
            $queryFilter = $this->evidenciaRepository->allEvidencias();
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar las evidencias: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getEvidenciaDetail(string $idEvidencia)
    {
        try {
            $queryFilter = $this->evidenciaRepository->detailEvidencia($idEvidencia);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar el detalle de la evidencia: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function isNivelRetoSuccessByEvidencias(NivelRetoDto $nivelRetoDto, string $idPlayer)
    {
        try {
            $idReto = $nivelRetoDto->idReto;
            $meta = $nivelRetoDto->meta;
            $tipoReto = $nivelRetoDto->tipoReto;
            $queryFilter = $this->evidenciaRepository->totaAmountsRegisterEvidencias($idPlayer, $idReto);
            $cantidadTotal = $queryFilter->cantidadTotal;
            $cantidadMonto = $queryFilter->cantidadMonto;
            $isNivelRetoSuccess = ($tipoReto === 'cantidad' && $cantidadTotal >= $meta) || ($tipoReto === 'dinero' && $cantidadMonto >= $meta);
            
            return new ResponseValidSuccess(true, $isNivelRetoSuccess);
        } catch (\Exception $e) {
            Log::error('Error al verificar si cumplio el nivel del reto: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseValidSuccess(false, false);
        }
    }

    public function getEvidenciasByPlayer(string $idPlayer)
    {
        try {
            $queryFilter = $this->evidenciaRepository->findEvidenciasByPlayer($idPlayer);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar las evidencias del participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getEvidenciasByUser(string $idUser)
    {
        try {
            $queryFilter = $this->evidenciaRepository->findEvidenciasByUser($idUser);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar las evidencias del usuario: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }
    

    public function addEvidencia(EvidenciaDto $evidenciaDto)
    {
        try {
            $queryAdd = $this->evidenciaRepository->createEvidencia($evidenciaDto);
            $evidenciaDto->id = $queryAdd->id;
            return new ResponseEvidenciaDto(true, $evidenciaDto);
        } catch (\Exception $e) {
            Log::error('Error al crear la evidencia: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseEvidenciaDto(false, null);
        }
    }

    public function updateEvidenciaEstatusById(EvidenciaDto $evidenciaDto,   string $idEvidencia){
        try {
            $envidenciaUpdateDto = $this->evidenciaRepository->updateEvidenciaEstatus($evidenciaDto, $idEvidencia);
            return new ResponseEvidenciaDto(true, $envidenciaUpdateDto);
        } catch (\Exception $e) {
            Log::error('Error al actualizar la evidencia: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseEvidenciaDto(false, null);
        }
    }

    public function addEvidenciaRegistro(EvidenciaRegistroDto $evidenciaRegistroDto)
    {
        try {
            $this->evidenciaRepository->createEvidenciaRegistro($evidenciaRegistroDto);
            return new ResponseEvidenciaRegistroDto(true, $evidenciaRegistroDto);
        } catch (\Exception $e) {
            Log::error('Error al crear el registro de evidencia: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseEvidenciaRegistroDto(false, null);
        }
    }

    public function addEvidenciaReto(EvidenciaRetoDto $evidenciaRetoDto)
    {
        try {
            $this->evidenciaRepository->createEvidenciaReto($evidenciaRetoDto);
            return true;
        } catch (\Exception $e) {
            Log::error('Error al crear el registro de evidencia para el reto: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }
}
