<?php

namespace App\Services;

use App\Dto\FileDto;
use App\Dto\Responses\ResponseDataDto;
use App\Dto\Responses\ResponseFileDto;
use App\Repositories\FileRepository;
use Illuminate\Support\Facades\Log;

class FileService
{
    protected $fileRepository;

    public function __construct(FileRepository $fileRepository)
    {
        $this->fileRepository = $fileRepository;
    }

    public function getFilesByUser(string $idUser)
    {
        try {
            $queryFilter = $this->fileRepository->allFilesByUser($idUser);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar los archivos del usuario: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getCountFilesWithRepeatName(string $nameSimple, string $extension, string $userId)
    {
        try {
            $queryFilter = $this->fileRepository->countFileWithNameRepeat($nameSimple, $extension, $userId);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al contar los archivos con el mismo nombre: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, 0);
        }
    }

    public function saveFile(FileDto $fileDto)
    {
        try {
            $newFileDto = $this->fileRepository->createFile($fileDto);
            return new ResponseFileDto(true, $newFileDto);
        } catch (\Exception $e) {
            Log::error('Error al guardar el archivo: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseFileDto(false, null);
        }
    }

    public function removeFile(string $idFile)
    {
        try {
            $this->fileRepository->deleteFile($idFile);
            return true;
        } catch (\Exception $e) {
            Log::error('Error al eliminar el archivo: ' . $e->getMessage(), ['exception' => $e, 'idFile' => $idFile ]);
            return false;
        }
    }
}
