<?php

namespace App\Services;

use App\Dto\NivelDto;
use App\Dto\Responses\ResponseDataDto;
use App\Dto\Responses\ResponseNivelDto;
use App\Repositories\NivelRepository;
use Illuminate\Support\Facades\Log;

class NivelService
{
    protected $nivelRepository;

    public function __construct(NivelRepository $nivelRepository)
    {
        $this->nivelRepository = $nivelRepository;
    }

    public function addNivelReto(NivelDto $nivelDto)
    {
        try {
            $newItem = $this->nivelRepository->createNivel($nivelDto);
            $nivelDto->id = $newItem->id;
            return new ResponseNivelDto(true, $nivelDto);
        } catch (\Exception $e) {
            Log::error('Error al crear el nivel: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseNivelDto(false, null);
        }
    }

    public function updateNivelReto(NivelDto $nivelDto)
    {
        try {
            $newItem = $this->nivelRepository->createNivel($nivelDto);
            $nivelDto->id = $newItem->id;
            return new ResponseNivelDto(true, $nivelDto);
        } catch (\Exception $e) {
            Log::error('Error al crear el nivel: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseNivelDto(false, null);
        }
    }

    public function createOrUpdateListNivels($niveles, string $idReto)
    {
        try {
            foreach ($niveles as $valor) {
                $nivelDto = new NivelDto();
                $nivelDto->id = $valor['id'];
                $nivelDto->nombre = $valor['nombre'];
                $nivelDto->meta = $valor['meta'];
                $nivelDto->puntos = $valor['puntos'];
                $nivelDto->fechaInicio = $valor['fecha_inicio'];
                $nivelDto->fechaFin = $valor['fecha_fin'];
                $nivelDto->insigniaId = $valor['insignia_id'];
                $nivelUpdate = $this->nivelRepository->findNivel($nivelDto->id);
                if ($nivelUpdate) {
                    $this->nivelRepository->updateNivel($nivelDto, $nivelDto->id);
                } else {
                    $nivelDto->retoId = $idReto;
                    $this->nivelRepository->createNivel($nivelDto);
                }
            }
            return true;
        } catch (\Exception $e) {
            Log::error('Error al crear o actualizar los niveles: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }

    public function removeListNivelsByReto(string $idReto, $nivelesReto)
    {
        try {
            $idFilterNotExitsNiveles = array_column($nivelesReto, 'id');
            $queryFilterNiveles = $this->nivelRepository->filterNivelGetDeleteWhereNotIds($idReto, $idFilterNotExitsNiveles);
            $nivelesQueryDelete = $queryFilterNiveles->toArray();
            if (!empty($nivelesQueryDelete)) {
                foreach ($nivelesQueryDelete as $valor) {
                    $idNivel = $valor['id'];
                    $nombreNivel = $valor['nombre'];
                    $resultDeleteNivel = $this->removeNivelById($idNivel);
                    if (!$resultDeleteNivel) {
                        return new ResponseDataDto(false, $nombreNivel);
                    }
                }
            }
            return new ResponseDataDto(true, null);
        } catch (\Exception $e) {
            Log::error('Error al eliminar los niveles del reto: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, "");
        }
    }

    public function removeNivelById(string $idNivel)
    {
        try {
            $this->nivelRepository->deleteNivel($idNivel);
            return true;
        } catch (\Exception $e) {
            Log::error('Error al eliminar el nivel del reto: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }

    public function getPointsTotalByPlayer(string $idPlayer)
    {
        try {
            $queryResult = $this->nivelRepository->totalPointsByPlayer($idPlayer);
            return new ResponseDataDto(true, $queryResult);
        } catch (\Exception $e) {
            Log::error('Error al consultar los puntos totales del participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getPointsWeekendTotalByPlayer(string $idPlayer)
    {
        try {
            $queryResult = $this->nivelRepository->totalPoinstWeekendByPlayer($idPlayer);
            return new ResponseDataDto(true, $queryResult);
        } catch (\Exception $e) {
            Log::error('Error al consultar los puntos por semana del participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getPointsMonthTotalByPlayer(string $idPlayer)
    {
        try {
            $queryResult = $this->nivelRepository->totalPoinstMonthByPlayer($idPlayer);
            return new ResponseDataDto(true, $queryResult);
        } catch (\Exception $e) {
            Log::error('Error al consultar los puntos por mes del participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getNivelFindById(string $idNivel)
    {
        try {
            $queryResult = $this->nivelRepository->findNivel($idNivel);
            return new ResponseNivelDto(true, $queryResult);
        } catch (\Exception $e) {
            Log::error('Error el detalle del nivel: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseNivelDto(false, null);
        }
    }

    public function getNivelsWhereNotRecompensaByEvidencia(string $idEvidencia, string $fechaCaptura)
    {
        try {
            $queryResult = $this->nivelRepository->findNivelsWhereNotRecompensaByEvidenciaId($fechaCaptura, $idEvidencia);
            return new ResponseDataDto(true, $queryResult);
        } catch (\Exception $e) {
            Log::error('Error al obtener los niveles que no tiene una recompensa relacionada: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }
}
