<?php

namespace App\Services;

use App\Dto\ProductDto;
use App\Dto\Responses\ResponseDataDto;
use App\Dto\Responses\ResponseProductDto;
use App\Repositories\ProductRepository;
use Illuminate\Support\Facades\Log;

class ProductService
{
    protected $productRepository;

    public function __construct(ProductRepository $productRepository)
    {
        $this->productRepository = $productRepository;
    }

    public function getAllProducts()
    {
        try {
            $queryAll = $this->productRepository->allProducts();
            return new ResponseDataDto(true, $queryAll);
        } catch (\Exception $e) {
            Log::error('Error al consultar los productos: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getProductById(int $idProduct)
    {
        try {
            $queryData = $this->productRepository->findProduct($idProduct);
            return new ResponseDataDto(true, $queryData);
        } catch (\Exception $e) {
            Log::error('Error al consultar la información del producto: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    

    public function getTotalVentaProductosByPlayer(int $idPlayer)
    {
        try {
            $queryData = $this->productRepository->findTotalVentaProductosByPlayer($idPlayer);
            return new ResponseDataDto(true, $queryData);
        } catch (\Exception $e) {
            Log::error('Error al consultar el total de venta de productos por participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getTotalVentaProductos()
    {
        try {
            $queryData = $this->productRepository->findTotalVentaProductos();
            return new ResponseDataDto(true, $queryData);
        } catch (\Exception $e) {
            Log::error('Error al consultar el total de venta por producto: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function addProduct(ProductDto $productDto)
    {
        try {
            $productNew = $this->productRepository->createProduct($productDto);
            $productDto->id = $productNew->id;
            return new ResponseProductDto(true, $productDto);
        } catch (\Exception $e) {
            Log::error('Error al crear producto: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseProductDto(false, null);
        }
    }

    public function updateProduct(ProductDto $productDto, int $idProduct)
    {
        try {
            $this->productRepository->updateProduct($productDto, $idProduct);
            return new ResponseProductDto(true, $productDto);
        } catch (\Exception $e) {
            Log::error('Error al actualizar el producto: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseProductDto(false, null);
        }
    }

    public function removeProductById(int $idProduct)
    {
        try {
            $this->productRepository->deleteProduct($idProduct);
            return true;
        } catch (\Exception $e) {
            Log::error('Error al eliminar el producto: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }
}
