<?php

namespace App\Services;

use App\Dto\RecompensaDto;
use App\Dto\Responses\ResponseDataDto;
use App\Dto\Responses\ResponseRecompensaDto;
use App\Repositories\RecompensaRepository;
use Illuminate\Support\Facades\Log;

class RecompensaService
{
    protected $recompensaRepository;

    public function __construct(RecompensaRepository $recompensaRepository)
    {
        $this->recompensaRepository = $recompensaRepository;
    }

    public function getRecompensasAll(){
        try {
            $queryAll = $this->recompensaRepository->allRecompensas();
            return new ResponseDataDto(true, $queryAll);
        } catch (\Exception $e) {
            Log::error('Error al consultar las recompensas: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getRecompensasActive(){
        try {
            $queryAll = $this->recompensaRepository->allRecompensasActive();
            return new ResponseDataDto(true, $queryAll);
        } catch (\Exception $e) {
            Log::error('Error al consultar las recompensas activas: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getRecompensaById(string $idRecompensa){
        try {
            $queryDetail = $this->recompensaRepository->detailRecompensa($idRecompensa);
            return new ResponseDataDto(true, $queryDetail);
        } catch (\Exception $e) {
            Log::error('Error al consultar el detalle de la recompensa: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getFindRecompensa(string $idRecompensa){
        try {
            $queryDetail = $this->recompensaRepository->findRecompensa($idRecompensa);
            $recompensaFind = new RecompensaDto();
            $recompensaFind->nombre = $queryDetail->nombre;
            $recompensaFind->metaCanje = $queryDetail->meta_canje;
            $recompensaFind->descripcionPremio = $queryDetail->descripcion_premio;
            $recompensaFind->estatus = $queryDetail->estatus;
            return new ResponseRecompensaDto(true, $recompensaFind);
        } catch (\Exception $e) {
            Log::error('Error al buscar la recompensa: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseRecompensaDto(false, null);
        }
    }
    
    public function addRecompensa(RecompensaDto $recompensaDto){
        try {
            $newItem = $this->recompensaRepository->createRecompensa($recompensaDto);
            $recompensaDto->id = $newItem->id;
            return new ResponseRecompensaDto(true, $recompensaDto);
        } catch (\Exception $e) {
            Log::error('Error al crear la recompensa: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseRecompensaDto(false, null);
        }
    }

    public function updateRecompensa(RecompensaDto $recompensaDto, int $idRecompensa){
        try {
            $this->recompensaRepository->updateRecompensa($recompensaDto, $idRecompensa);
            return new ResponseRecompensaDto(true, $recompensaDto);
        } catch (\Exception $e) {
            Log::error('Error al actualizar la recompensa: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseRecompensaDto(false, null);
        }
    }

    public function removeRecompensaById(int $idRecompensa){
        try {
            $this->recompensaRepository->deleteRecompensa($idRecompensa);
            return true;
        } catch (\Exception $e) {
            Log::error('Error al eliminar la recompensa: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }
}
