<?php

namespace Tests\Unit;

use App\Dto\PlayerDto;
use App\Dto\Responses\ResponsePlayerDto;
use App\Models\File;
use App\Models\Participante;
use App\Models\User;
use App\Repositories\PlayerRepository;
use App\Services\PlayerService;
use Exception;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\Log;
use Mockery;
use Tests\TestCase;

class ParticipanteTest extends TestCase
{
    use RefreshDatabase, WithFaker;

    public function tearDown(): void
    {
        parent::tearDown();
        Mockery::close();
    }

    public function test_create_player()
    {
        $user = User::create([
            'email' => $this->faker->email,
            'password' => $this->faker->password
        ]);

        $fileData = [
            'nombre' => $this->faker->name,
            'path' => '',
            'type' => $this->faker->mimeType,
            'size' => $this->faker->numberBetween(1000, 10000),
            'user_id' => $user->id
        ];

        $file = File::create($fileData);

        $playerDto = new PlayerDto("Aldair Reyes Sánchez", "", "", "", "", "", "", $file->id, $user->id, "");

        $participanteMock = Participante::create([
            "nombre_tienda" => $playerDto->nombreTienda,
            "direccion" => $playerDto->direccion,
            "nombre_completo" => $playerDto->nombreCompleto,
            "localidad" => $playerDto->localidad,
            "estado" => $playerDto->estado,
            "ciudad" => $playerDto->ciudad,
            "ruta" => $playerDto->ruta,
            "file_id" => $playerDto->fileId,
            "user_id" => $playerDto->userId,
            "identificador" => $playerDto->identificador
        ]);

        $playerRepositoryMock = Mockery::mock(PlayerRepository::class);
        $this->app->instance(PlayerRepository::class, $playerRepositoryMock);
        $playerRepositoryMock
            ->shouldReceive('createPlayer')
            ->once()
            ->with($playerDto)
            ->andReturn($participanteMock);

        // $playerRepositoryMock = $this->getMockBuilder(PlayerRepository::class)
        //     ->disableOriginalConstructor()
        //     ->getMock();

        // $playerRepositoryMock->expects($this->once())
        //     ->method('createPlayer')
        //     ->with($playerDto)
        //     ->willReturn($participanteMock);

        // $playerService = new PlayerService($this->playerRepositoryMock);
        // $response = $playerService->addPlayer($playerDto);
        // $this->assertTrue($response->success);
        // $this->assertEquals($playerDto, $response->player);
    }
}
